#!/usr/bin/env python3

from math import degrees, radians

from magvar import magnetic_variation

files = 0

USAGE = "Usage: python3 test_values.py {TestValues text file}..."

def main(args):
    errors = 0

    if len(args) == 0:
        usage()

    print('TAP version 13')
    print('1..%d' % len(args))

    for fn in args:
        test_file(fn)

def test_file(fn):
    global files

    errors = count = 0

    files += 1

    with open(fn) as f:
        for line in f:
            a = line.split()

            if len(a) == 18:
                a = [float(x) for x in a]

                mv = magnetic_variation(a[0],          # Date
                                        radians(a[2]), # geodetic latitude
                                        radians(a[3]), # geodetic longitude
                                        a[1] * 1000)   # Height above ellipsoid

                error = abs(a[4] + degrees(mv))

                count += 1

                if error > 0.01:
                    print('not ok', files, '- expected', a[4], 'got', -degrees(mv))
                    errors += 1
                    break

    if not errors:
        print('ok 1 -', files, 'tests passed')

def usage():
    from sys import stderr, exit

    print(USAGE, file=stderr)
    exit(1)

if __name__ == '__main__':
    from sys import argv
    main(argv[1:])



#!/usr/bin/env python3
# https://www.wayforward.net/magvar/julian.py
#
# Copyright(c) 2024, Wayforward Technology, LLC
#
# Author: Terence Way <terry@wayforward.net>
#
"""Convert Python date and datetime objects into a floating-point Julian
time, suitable for passing into the magvar.magnetic_variation function.
"""

from datetime import datetime

def is_leap(year):
    """True iff _year_ is a leap year:

    >>> is_leap(1992)
    True

    >>> is_leap(1993)
    False

    >>> is_leap(1999)
    False

    >>> is_leap(2000)
    True

    >>> is_leap(2100)
    False
    """
    return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)

def days_in_year(year):
    """Returns 366 iff year is a leap year.  Returns 365 otherwise.

    >>> days_in_year(2000)
    366

    >>> days_in_year(2100)
    365

    >>> days_in_year(2021)
    365

    >>> days_in_year(2024)
    366
    """
    return [365, 366][int(is_leap(year))]

def julian(dt):
    """
    Returns floating point "julian" date.

    >>> julian(datetime(2022, 1, 1))
    2022.0

    >>> round(julian(datetime(2022, 1, 2)), 4)
    2022.0027

    >>> round(julian(datetime(2022, 12, 31)), 4)
    2022.9973
    """
    if hasattr(dt, 'timetuple'):
        t = dt.timetuple()
        y = t.tm_year
        return y + (t.tm_yday - 1) / days_in_year(y)

    return float(dt)

def _test():
    from doctest import testmod
    return testmod()

if __name__ == '__main__':
    _test() 



#!/usr/bin/env python3

from distutils.core import setup, Extension

magvar_module = Extension('magvar',
			  sources=['av_mag.c', 'magvarmodule.c', 'wmm.c'])

setup(name='magvar',
      version='1.1',
      description='Magnetic variation (declination) at a given date, latitude, longitude, and elevation',
      ext_modules = [magvar_module],
      author='Terence Way',
      author_email='terry@wayforward.net')
